package net.laparola.ui;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.EnumSet;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.laparola.core.RicercaErroreSintassiException;
import net.laparola.core.RicercaEspressioneVuotaException;
import net.laparola.core.RicercaParentesiException;
import net.laparola.core.RicercaParentesiQuadrateException;
import net.laparola.core.Riferimento;
import net.laparola.core.Testi.TestoTipi;
import net.laparola.core.TestoNonEsisteException;

public class LaParolaUrl {
	// da escludere : *?|!/\^~()[]:"<>
	private static final Pattern url_regex = Pattern.compile("^(?:(.+?):)?/*((.*?)(?:_(.*?))?(?:@(.*?))?(?:#(.*?))?)$");

	public String requesturl;
	public String schema;
	public String contenuto;
	public String brani;
	public String ricerca;
	public String versione;
	public String ancoraggio;
	public String destinazione;
	public boolean forzaAltraFinestra;
	public boolean cacheAttiva;
	public boolean gestito;
	public Riferimento riferimento;
	public String testo;
	public int[] lcv;


	/* package */LaParolaUrl(String schema, String parte_gerarchica, String contenuto, String ricerca, String versione, String ancoraggio, LaParolaBrowser browser) {
		this.schema = schema;
		this.contenuto = parte_gerarchica;
		this.brani = contenuto;
		this.ricerca = ricerca;
		this.versione = versione;
		this.ancoraggio = ancoraggio;

		gestito = true;

		init(browser.getVersione());
		
		requesturl = getUrl();
	}

	/* package */LaParolaUrl(String url, String versionePredefinita) {
		try {
			url = URLDecoder.decode(url, "UTF-8");
		} catch (UnsupportedEncodingException e) {
		}
		
		requesturl = url;
		
		Matcher m = url_regex.matcher(url);
		if (!m.matches()) {
			gestito = false;
		} else {
			gestito = true;

			schema = m.group(1);
			contenuto = m.group(2);
			brani = m.group(3);
			ricerca = m.group(4);
			versione = m.group(5);
			ancoraggio = m.group(6);

			init(versionePredefinita);
		}
	}

	private void init(String versionePredefinita) {
		riferimento = null;

		if (schema == null || schema.length() == 0)
			schema = "laparola";

		// if (contenuto != null && contenuto.length() == 0)
		// contenuto = null;

		if (brani != null && brani.length() == 0)
			brani = null;

		if (ricerca != null && ricerca.length() == 0)
			ricerca = null;

		if (versione == null || versione.length() == 0)
			versione = versionePredefinita != null ? versionePredefinita : "";
		
		if (versione.toLowerCase().equals("*bibbia")) {
			versione = LaParolaBrowser.getUltimaBibbia();
		}

		if (ancoraggio == null || ancoraggio.length() == 0)
			ancoraggio = "inizio";

		if (schema.equals("laparola")) {
			cacheAttiva = true;
			creaRiferimento();
		} else if (schema.equals("lpfile")) {
			cacheAttiva = false;
		} else if (schema.equals("lpcomando")) {
			cacheAttiva = false;
		} else if (schema.equals("lpsegnalibri")) {
			cacheAttiva = false;
		} else if (schema.equals("lpsegnalibro")) {
			cacheAttiva = false;
		} else if (schema.equals("lppreferiti")) {
			cacheAttiva = false;
		} else if (schema.equals("lpcronologia")) {
			cacheAttiva = false;
		} else if (schema.equals("null")) {
			gestito = false;
		} else {
			gestito = false;
			//throw new RuntimeException("Schema non gestito (" + schema + ")");
			schema = "error";
		}
	}

	private void creaRiferimento() {
		if (brani != null && ricerca == null) {
			if (!brani.startsWith("$")) {
				riferimento = LaParolaBrowser.creaRiferimento(brani, versione);
				brani = LaParolaBrowser.normalizzaRiferimento(riferimento, versione);
	
				if (ancoraggio != null) {
					try {
						String[] s = ancoraggio.split("_");
	
						int b = 1;
						for (int i = 1; i <= 73; i++) {
							if (LaParolaBrowser.getAbbreviazioneLibro(i).equals(s[0])) {
								b = i;
								break;
							}
						}
	
						lcv = new int[] { b, Integer.parseInt(s[1]), Integer.parseInt(s[2]) };
					} catch (Exception e) {
						//
					}
				}
	
				if (lcv == null) {
					List<int[]> rifbrani = riferimento.getBrani();
					// se è un riferimento continuo, va all'inizio, indipendentemente da eventuali libri o capitoli diversi
					if (rifbrani.size() == 1) {
						int[] rifbrano = rifbrani.get(0);
						// if (rifbrano[0] == rifbrano[3] && rifbrano[1] == rifbrano[4]) {
						lcv = new int[] { rifbrano[0], rifbrano[1], rifbrano[2] };
						// }
					}
				}
			} else {
				// è un riferimento ad una nota con titolo
				riferimento = null;
				testo = null;
			}
		} else if (ricerca != null) {
			try {
				brani = LaParolaBrowser.normalizzaRiferimento(brani, versione);
			} catch (Exception e) {
				//
			}

			try {
				riferimento = LaParolaBrowser.ricerca(ricerca, brani != null ? brani : "", versione);
			} catch (RicercaEspressioneVuotaException e) {
				testo = LaParolaBrowser.Stringhe.get(LaParolaStringhe.ERRORE_RICERCA_ESPRESSIONE_VUOTA);
				cacheAttiva = false;
			} catch (RicercaErroreSintassiException e) {
				if (brani != null || !LaParolaBrowser.isRiferimento(ricerca, versione)) {
					testo = String.format(LaParolaBrowser.Stringhe.get(LaParolaStringhe.ERRORE_RICERCA_ERRORE_SINTASSI), e.getMessage());
				} else {
					testo = String.format(LaParolaBrowser.Stringhe.get(LaParolaStringhe.ERRORE_RICERCA_ERRORE_SINTASSI_PROPONI_RIFERIMENTO), e.getMessage(), ricerca);
				}
				cacheAttiva = false;
			} catch (RicercaParentesiException e) {
				testo = LaParolaBrowser.Stringhe.get(LaParolaStringhe.ERRORE_RICERCA_PARENTESI);
				cacheAttiva = false;
			} catch (RicercaParentesiQuadrateException e) {
				testo = LaParolaBrowser.Stringhe.get(LaParolaStringhe.ERRORE_RICERCA_PARENTESI_QUADRATE);
				cacheAttiva = false;
			} catch (Exception e) {
				if (brani != null || !LaParolaBrowser.isRiferimento(ricerca, versione)) {
					testo = LaParolaBrowser.Stringhe.get(LaParolaStringhe.ERRORE_RICERCA);
				} else {
					testo = String.format(LaParolaBrowser.Stringhe.get(LaParolaStringhe.ERRORE_RICERCA_PROPONI_RIFERIMENTO), ricerca);
				}
				cacheAttiva = false;
			}
		} else {
			gestito = false;
		}
	}

	public String getChiaveCache() {
        return String.format("%s:%s_%s@%s", 
    		schema, 
    		brani != null ? brani : "", 
			ricerca != null ? ricerca : "",
			versione);
	}

	public boolean stessoTestoDi(LaParolaUrl altro) {
		return altro != null && getChiaveCache().equals(altro.getChiaveCache());
	}

	public String getUrl() {
		return getUrl(null, true);
	}

	public String getUrlConAltraVersione(String nuovaVersione) {
		return getUrl(nuovaVersione, false);
	}
	
	public String getUrlConAltraVersione(String nuovaVersione, boolean conSegnalibro) {
		return getUrl(nuovaVersione, conSegnalibro);
	}

	private String getUrl(String nuovaVersione, boolean conSegnalibro) {
		if (schema.equals("lpcomando")) {
			return String.format("lpcomando:%s", contenuto);
		}

		String nuoviBrani = brani;
		if (schema.equals("laparola") && riferimento != null) {
			if (nuovaVersione != null && brani != null && versione != null) {
				nuoviBrani = LaParolaBrowser.cambiaVersioneRiferimento(riferimento, versione, nuovaVersione);
			}
		}

        return String.format("%s:%s_%s@%s#%s", 
    		schema, 
    		nuoviBrani != null ? nuoviBrani : "", 
			ricerca != null ? ricerca : "",
			nuovaVersione == null ? (versione != null ? versione : "") : nuovaVersione,
			(ancoraggio != null && conSegnalibro) ? ancoraggio : "");
	}

	public String getDescrizione() {
		String v = versione != null ? versione : "";

		if (schema.equals("laparola")) {
			if (ricerca != null) {
				if (brani == null) {
					return LaParolaBrowser.Stringhe.get(LaParolaStringhe.DESCRIZIONE_URL_RICERCA, ricerca, v);
				}
				return LaParolaBrowser.Stringhe.get(LaParolaStringhe.DESCRIZIONE_URL_RICERCA_IN, ricerca, brani, v);
			}
			return LaParolaBrowser.Stringhe.get(LaParolaStringhe.DESCRIZIONE_URL_BRANI, brani, v);
		} else if (schema.equals("lpfile")) {
			int ipos = brani.lastIndexOf('/');
			int fpos = brani.lastIndexOf('.');

			if (ipos == -1)
				ipos = -1;
			if (fpos == -1)
				fpos = brani.length() + 1;

			return LaParolaBrowser.Stringhe.get(LaParolaStringhe.DESCRIZIONE_URL_FILE, brani.substring(ipos + 1, fpos), v);
		} else if (schema.equals("lpsegnalibri")) {
			return LaParolaBrowser.Stringhe.get(LaParolaStringhe.DESCRIZIONE_URL_GRUPPI_SEGNALIBRI);
		} else if (schema.equals("lpsegnalibro")) {
			return LaParolaBrowser.Stringhe.get(LaParolaStringhe.DESCRIZIONE_URL_GRUPPO_SEGNALIBRI, brani, v);
		}

		return "";
	}

	public String getUrlSuccessivo() {
		if (!schema.equals("laparola") || riferimento == null) {
			return null;
		}

		List<int[]> rifbrani = riferimento.getBrani();
		if (rifbrani.size() != 1) {
			return null;
		}
		String versioneDaControllare = versione;

		synchronized (LaParolaBrowser.mTesti) {
			try {
				if (!LaParolaBrowser.mTesti.getInfo(versione).getTipo().contains(TestoTipi.BIBBIA)) {
					versioneDaControllare = LaParolaBrowser.getUltimaBibbia();
					if (versioneDaControllare.equals(""))
						return null;
				}
			} catch (TestoNonEsisteException e) {
				return null;
			}

			String rif = null;

			int[] rifbrano = rifbrani.get(0);
			int l1 = rifbrano[0];
			int c1 = rifbrano[1];
			int l2 = rifbrano[3];
			int c2 = rifbrano[4];
			int v2 = rifbrano[5];
			int versettiInUltimoCapitolo = LaParolaBrowser.mTesti.versettiInCapitolo(l2, c2, versioneDaControllare);

			if (v2 >= versettiInUltimoCapitolo) {
				// alla fine del capitolo : vado al capitolo successivo
				if (c2 >= LaParolaBrowser.mTesti.capitoliInLibro(l2, versioneDaControllare)) {
					// ultimo capitolo di un libro : vado al libro successivo
					int l = l2 + 1;
					while (l < 74 && LaParolaBrowser.mTesti.capitoliInLibro(l, versioneDaControllare) == 0) {
						// considero i libri mancanti nelle varie versioni
						l++;
					}
					if (l < 74) { // altrimenti siamo all'ultimo libro del testo: Apoc, o Malachia se solo AT, ecc
						int c = 1;
			    		
			            rif = String.format("laparola:%d %d %d %d %d %d@%s#inizio",
			            		l, c, 1, l, c, 255,
			                    versione);
					}
				} else {
					// altro capitolo
			        rif = String.format("laparola:%d %d %d %d %d %d@%s#inizio",
			                l1, c1 + 1, 1, l1, c1 + 1, 255,
			                versione);
				}

			} else {
				// in mezzo al capitolo : mostra l'intero capitolo (eventualmente, gli interi capitoli)
	            rif = String.format("laparola:%d %d %d %d %d %d@%s#%s_%d_%d",
	                    l1, c1, 1, l2, c2, 255,
	                    versione,
	                    LaParolaBrowser.mTesti.getLibroAbbreviazioneUsata(l2), c2, v2 + 1);
			}

			return rif;
		}
	}

	public String getUrlPrecedente() {
		if (!schema.equals("laparola") || riferimento == null) {
			return null;
		}

		List<int[]> rifbrani = riferimento.getBrani();
		if (rifbrani.size() != 1) {
			return null;
		}
		String versioneDaControllare = versione;

		synchronized (LaParolaBrowser.mTesti) {
			try {
				if (!LaParolaBrowser.mTesti.getInfo(versione).getTipo().contains(TestoTipi.BIBBIA)) {
					versioneDaControllare = LaParolaBrowser.getUltimaBibbia();
					if (versioneDaControllare.equals(""))
						return null;
				}
			} catch (TestoNonEsisteException e) {
				return null;
			}

			String rif = null;

			int[] rifbrano = rifbrani.get(0);
			int l1 = rifbrano[0];
			int c1 = rifbrano[1];
			int v1 = rifbrano[2];
			int l2 = rifbrano[3];
			int c2 = rifbrano[4];
			// int v2 = rifbrano[5];
			int versettiInUltimoCapitolo = LaParolaBrowser.mTesti.versettiInCapitolo(l2, c2, versioneDaControllare);

			if (v1 == 1) {
				// inizio capitolo : vai a capitolo precedente
				if (c1 == 1) {
					// primo capitolo di un libro : torna al libro precedente
					int l = l1 - 1;
					while (l > 0 && LaParolaBrowser.mTesti.capitoliInLibro(l, versioneDaControllare) == 0) {
						// considero i libri mancanti nelle varie versioni
						l--;
					}
					if (l > 0) { // altrimenti siamo al primo libro del testo: Genesi, o Matteo se solo NT, ecc
						int c = LaParolaBrowser.mTesti.capitoliInLibro(l, versioneDaControllare);

			            rif = String.format("laparola:%d %d %d %d %d %d@%s#inizio",
			            		l, c, 1, l, c, versettiInUltimoCapitolo,
			                    versione);
					}
				} else {
					// altro capitolo
			        rif = String.format("laparola:%d %d %d %d %d %d@%s#inizio",
			                l1, c1 - 1, 1, l1, c1 - 1, 255,
			                versione);
				}
			} else {
				// in mezzo al capitolo : mostra l'intero capitolo (eventualmente, gli interi capitoli)
			    rif = String.format("laparola:%d %d %d %d %d %d@%s#inizio",
			            l1, c1, 1, l2, c2, 255,
			            versione);
			}

			return rif;
		}
	}

	@Override
	public boolean equals(Object o) {
		if (!(o instanceof LaParolaUrl)) {
			return false;
		}
		LaParolaUrl other = (LaParolaUrl) o;

		return getUrl().equals(other.getUrl());
	}

	public String getUrlSenzaAncoraggio() {
		return getUrl(null, false);
	}

	public boolean richiedeNuovaFinestra(LaParolaUrl urlCorrente) {
		return forzaAltraFinestra ||
			   (urlCorrente.getTipoTesto().contains(TestoTipi.COMMENTARIO) && 
					   this.getTipoTesto().contains(TestoTipi.BIBBIA));
	}
	
	public EnumSet<TestoTipi> getTipoTesto() {
		try {
			return LaParolaBrowser.getInformazioniTesto(versione).getTipo();
		} catch (Exception e) {
			return EnumSet.of(TestoTipi.NESSUNO);
		}
	}
}