package net.laparola.ui.android;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import net.laparola.R;
import net.laparola.core.VersioneInformazioni;
import net.laparola.ui.LaParolaBrowser;
import net.laparola.ui.android.dialogs.HoloDialog;
import net.laparola.ui.android.dialogs.MessageDialog;
import net.laparola.ui.android.library.LibraryActivity;
import net.laparola.ui.utils.LZMAFile;
import net.laparola.ui.utils.lzma_java.LZMADecoder;
import android.annotation.SuppressLint;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.DialogInterface.OnDismissListener;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager.NameNotFoundException;
import android.os.Environment;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.widget.LinearLayout;

public class LaParolaActivityInitUtility {
	private LaParolaActivity parent;
	private WebView messageView;
	private boolean mIsWorking;

	public LaParolaActivityInitUtility(LaParolaActivity parent) {
		this.parent = parent;
		mIsWorking = true;
	}

	@SuppressLint("SetJavaScriptEnabled") 
	private boolean checkBibleInstalled(final String path) {
		final String nomeVersioneInRisorse = "Nuova Riveduta";
		int v1 = 1;
		int v2 = 1;
		int v3 = 0;

		VersioneInformazioni ivi = LaParolaBrowser.getInformazioniTesto(nomeVersioneInRisorse);
		boolean installataAggiornabile = (ivi != null && (
			(v1 > ivi.getVersione1()) || 
			(v1 == ivi.getVersione1() && v2 > ivi.getVersione2()) || 
			(v1 == ivi.getVersione1() && v2 == ivi.getVersione2() && v3 > ivi.getVersione3())));

		if (installataAggiornabile) {
			LaParolaBrowser.cancellaTesto(nomeVersioneInRisorse, ivi.getNomeDelFile());
		}

		// installataAggiornabile = true;
		
		if (installataAggiornabile || LaParolaBrowser.getNomiVersioni().length == 0) {
			parent.runOnUiThread(new Runnable() {
				@Override
				public void run() {
					messageView = new WebView(parent);
					LinearLayout activityLayout = (LinearLayout)parent.findViewById(R.id.main_activity_container);
					activityLayout.addView(messageView, 0, new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.MATCH_PARENT));
					WebSettings mSettings = messageView.getSettings();
					mSettings.setJavaScriptEnabled(true);
					mSettings.setBuiltInZoomControls(false);
					messageView.loadUrl("file:///android_asset/preparazione.html");
				}
			});
			
			Thread resourceCopyingThread = new Thread(new Runnable() {
				@Override
				public void run() {
					String nomeRisorsa = nomeVersioneInRisorse.toLowerCase().replace(' ', '_');
					String nomeFile = path + "/" + nomeVersioneInRisorse + ".lpj";

					copyBibleFromResource(nomeRisorsa, nomeFile);

					parent.runOnUiThread(new Runnable() {
						@Override
						public void run() {
							checkBibleFromResourceCopied(path);
						}
					});
				}
			});

			resourceCopyingThread.start();
			
			return false;
		} else {
			return true;
		}
	}

	private void checkBibleFromResourceCopied(final String path) {
		LaParolaBrowser.aggiungiTestiDaDirectory(path);
		LinearLayout activityLayout = (LinearLayout)parent.findViewById(R.id.main_activity_container);
		activityLayout.removeView(messageView);
		messageView = null;

		mIsWorking = false;

		if (LaParolaBrowser.getNomiVersioni().length == 0) {
			parent.runOnUiThread(new Runnable() {
				@Override
				public void run() {
					HoloDialog d = new MessageDialog(parent, R.string.error, R.string.no_bible_installed);
					d.setOnDismissListener(new OnDismissListener() {
						@Override
						public void onDismiss(DialogInterface dialog) {
							parent.startActivity(new Intent(parent, LibraryActivity.class));
						}
					});
					d.show();
				}
			});
		}
	}

	private boolean checkStoragePath(String storagePath) {
		File f = new File(storagePath + "/tmp");
		f.delete();
		try {
			f.createNewFile();
		} catch (IOException e) {
			HoloDialog d = new MessageDialog(parent, R.string.error, R.string.storage_error);
			d.setOnDismissListener(new OnDismissListener() {
				@Override
				public void onDismiss(DialogInterface dialog) {
					parent.finish();
				}
			});
			d.show();
			return false;
		}
		f.delete();
		return true;
	}

	private void copyBibleFromResource(String resourceName, String fileName) {
		// È necessario usare questo metodo perchè gli asset non permettono l'accesso casuale e
		// prima di android 2.3 non possono superare quando decompressi dall'apk la dimensione di 1 MB.
		// Però l'eccezione è per alcuni tipi di risorse (con certi estensioni) che non vengono compressi dall'apk.
		// Quindi noi dobbiamo comprimere la risorsa con zip (altrimenti il file apk è troppo grande),
		// e poi dare un'estensione come mp3 affinchè non venga compresa dentro l'apk.
		// Vedi per esempio http://ponystyle.com/blog/2010/03/26/dealing-with-asset-compression-in-android-apps/

		// così può essere compilato anche togliendo il file da raw
		int id = parent.getResources().getIdentifier(resourceName, "raw", parent.getApplicationInfo().packageName);
		if (id == 0) {
			return;
		}

		File file = new File(fileName);
		file.mkdirs();
		file.delete(); // makedirs crea una cartella chiamata nomefile

		final int BUFFER = 8192;
		BufferedInputStream src = new BufferedInputStream(parent.getResources().openRawResource(id), BUFFER);
		BufferedOutputStream dest = null;
		try {
			FileOutputStream openFileOutput = new FileOutputStream(file);
			dest = new BufferedOutputStream(openFileOutput, BUFFER);
		} catch (Exception e) {
			return;
		}

		// ZipInputStream zis = null;
		try {
			/*
			 * zis = new ZipInputStream(src); zis.getNextEntry(); int count; byte data[] = new byte[BUFFER]; while ((count = zis.read(data, 0, BUFFER)) != -1) dest.write(data, 0,
			 * count);
			 */
			LZMAFile.decomprimi(src, dest, new LZMADecoder.ProgressRunnable() {
				int lastPercent = 0;
				long lastPercentTime = 0;

				@Override
				public void publish(long progress, long size) {
					final int percent = (int) Math.round((double) progress / size * 100);
					long milliTime = System.nanoTime() / 1000000;
					if (lastPercent != percent && milliTime > lastPercentTime + 1000) {
						parent.runOnUiThread(new Runnable() {
							@Override
							public void run() {
								String command = "publishProgress(" + String.valueOf(percent) + ");";
								messageView.loadUrl("javascript:(function() {" + command + "})()");
							}
						});
						lastPercent = percent;
						lastPercentTime = milliTime;
					}
				}
			});
		} catch (Exception e) {
			file.delete();
			
			MessageDialog d = new MessageDialog(parent, R.string.error, R.string.error_decompressing_resource);
			d.show();
		} finally {
			try {
				// if (dest != null) {
				dest.flush();
				dest.close();
				// }
			} catch (Exception e) {
				//
			}
			/*
			 * try { if (zis != null) zis.close(); } catch (Exception e) {}
			 */
		}
	}

	public boolean isWorking() {
		return mIsWorking;
	}

	public void init() {
		new Thread(new Runnable() {
			@Override
			public void run() {
				initThread();
			}
		}).start();
	}

	private void initThread() {
		int v1, v2;
		try {
			PackageInfo manager = parent.getPackageManager().getPackageInfo(parent.getPackageName(), 0);
			String[] p = manager.versionName.split("\\.");
			v1 = Integer.parseInt(p[0]);
			v2 = Integer.parseInt(p[1]);
		} catch (NameNotFoundException e) {
			v1 = 1;
			v2 = 0;
		}
		LaParolaBrowser.inizializza(v1, v2);
		LaParolaBrowser.setLaParolaBrowserStaticClient(parent);

		String storagePath = parent.sharedPreferences.getString("storagePath", "");
		if (storagePath.length() == 0) {
			// path = getFilesDir().getAbsolutePath();
			storagePath = Environment.getExternalStorageDirectory().getPath() + "/laparola";

			SharedPreferences.Editor editor = parent.sharedPreferences.edit();
			editor.putString("storagePath", storagePath);
			editor.commit();
		}
		(new File(storagePath)).mkdirs();

		if (checkStoragePath(storagePath) == false)
			return;

		if (LaParolaBrowser.getGruppiSegnalibri().isEmpty()) {
			LaParolaBrowser.aggiungiSegnalibriDaFile("segnalibri.xml");
			LaParolaBrowser.aggiungiPreferitiDaFile(storagePath + "/preferiti.xml");
			LaParolaBrowser.caricaCronologia(storagePath + "/cronologia");
		}

		LaParolaBrowser.aggiungiTestiDaDirectory(storagePath);
		
		if (checkBibleInstalled(storagePath)) {
			mIsWorking = false;
		}
	}
}