package net.laparola.ui.android.actionbar;

import java.util.HashMap;
import java.util.List;
import java.util.Locale;

import com.actionbarsherlock.view.MenuItem;
import com.actionbarsherlock.view.MenuItem.OnActionExpandListener;
import net.laparola.R;
import net.laparola.ui.android.LaParolaActivity;
import net.laparola.ui.android.LaParolaFragment;
import net.laparola.ui.android.LaParolaFragment.AnchorAndText;
import net.laparola.ui.android.actionbar.view.IgnPopupWindow;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.drawable.Drawable;
import android.media.AudioManager;
import android.speech.tts.TextToSpeech;
import android.view.*;
import android.view.View.*;
import android.widget.*;
import android.widget.SeekBar.OnSeekBarChangeListener;

public class TTSActionItemManager implements OnActionExpandListener, OnClickListener, TextToSpeech.OnInitListener, TextToSpeech.OnUtteranceCompletedListener {
    private static final int HUGE = 30000;
    
	private LaParolaActivity parent;
    private MenuItem ttsMenuItem;
    private View readButton;
    private View stopButton;
    private View rewindButton;
    private View ffButton;
    private View settingsButton;
	private TextToSpeech mTts;
	private List<AnchorAndText> mPlayList;
	private int mPlayListIndex;
	private TTSSettingsPopup mPopup;
    
	public TTSActionItemManager(LaParolaActivity parent, MenuItem item) {
        this.parent = parent;
        this.ttsMenuItem = item;
		LinearLayout searchActionView = (LinearLayout)ttsMenuItem.getActionView();
        
        readButton = (View)searchActionView.findViewById(R.id.tts_read_button);
        readButton.setOnClickListener(this);
        
        stopButton = (View)searchActionView.findViewById(R.id.tts_stop_button);
        stopButton.setOnClickListener(this);
        
        rewindButton = (View)searchActionView.findViewById(R.id.tts_rewind_button);
        rewindButton.setOnClickListener(this);
        
        ffButton = (View)searchActionView.findViewById(R.id.tts_ff_button);
        ffButton.setOnClickListener(this);
        
        settingsButton = (View)searchActionView.findViewById(R.id.tts_settings_button);
        settingsButton.setOnClickListener(this);

        setButtonsVisibility();
        
        mPopup = new TTSSettingsPopup(parent);
        mPopup.setBackgroundDrawable(parent.getResources().getDrawable(R.drawable.abs__menu_dropdown_panel_holo_light));
        
		ttsMenuItem.setOnActionExpandListener(this);
	}

	private void readTTS() {
		if (mTts == null) {
			mTts = new TextToSpeech(parent, this);
		} else {
        	LaParolaFragment activeFragment = parent.getActiveFragment();
        	if (activeFragment == null)
        		return;
        		
			String lingua = activeFragment.getInformazioniVersione().getLingua();
        	Locale locale = new Locale(lingua);
        	int result = mTts.setLanguage(locale);
        	
            if (result == TextToSpeech.LANG_MISSING_DATA || result == TextToSpeech.LANG_NOT_SUPPORTED) {
            	String s = parent.getString(R.string.tts_unsupported_language, "\"" + lingua + "\"");
                Toast.makeText(parent, s, Toast.LENGTH_LONG).show();
                startInstallLanguage();
                return;
            }
            
			initTextToRead();
			readCurrent();
		}
	}
	
	private void readCurrent () {
		synchronized (this) {
			if (mPlayList == null)
				return;
			
			setButtonsVisibility();
			
			if (mPlayListIndex < 0) {
				// sono tornato troppo indietro
				mPlayListIndex = 0;
			} else if (mPlayListIndex >= mPlayList.size()) {
				// ho finito, o sono andato troppo avanti
				return;
			}
			
			String s = mPlayList.get(mPlayListIndex).text;

			// queste correzioni servono ad evitare errori del sintetizzatore 
			s = s.replace('«', '"');
			s = s.replace('»', '"');
			s = s.replace(" - ", ", ");
			
			HashMap<String, String> params = new HashMap<String, String>();
			params.put(TextToSpeech.Engine.KEY_PARAM_UTTERANCE_ID, "1");   // altrimenti non viene chiamato
																		   // onUtteranceCompleted
			
			mTts.speak(s, TextToSpeech.QUEUE_FLUSH, params);
		}
	}

	private void setButtonsVisibility() {
		final int rv, sv;
		if (mPlayList == null || mPlayListIndex < 0 || mPlayListIndex >= mPlayList.size()) {
			rv = View.VISIBLE;
			sv = View.GONE;
		} else {
			rv = View.GONE;
			sv = View.VISIBLE;
		}
			
		readButton.post(new Runnable() {
			@Override
			public void run() {
				readButton.setVisibility(rv);
				rewindButton.setVisibility(sv);
				stopButton.setVisibility(sv);
				ffButton.setVisibility(sv);
			}
		});	
	}

	private void initTextToRead() {
		synchronized (this) {
			LaParolaFragment activeFragment = parent.getActiveFragment();
			mPlayList = activeFragment.getPlainTextWithAnchors();
			String currentAnchor = "#" + activeFragment.getUrlCorrente().ancoraggio;
			
			mPlayListIndex = -1;
			
			for (int i = 0; i < mPlayList.size(); i++) {
				AnchorAndText v = mPlayList.get(i);
				if (v.link != null && v.link.endsWith(currentAnchor)) {
					mPlayListIndex = i;
					break;
				}
			}
			
			if (mPlayListIndex == -1) {
				mPlayListIndex = 0;
			}
		}
	}

	private void showTTSSettings () {
        if (!mPopup.isShowing()) {
        	mPopup.show();
        }
	}
	
    public void onClick(View view) {
    	if (view == readButton) {
    		readTTS();
    	} else if (view == stopButton) {
    		stop();
    	} else if (view == rewindButton) {
    		mPlayListIndex--;
    		readCurrent();
    	} else if (view == ffButton) {
    		if (mTts != null) {
    			mTts.stop();
    		}
    		// va avanti in automatico perché viene
    		// chiamato onUtteranceCompleted
    	} else if (view == settingsButton) {
    		showTTSSettings();
    	}
    }

	public void stop() {
		if (mTts != null) {
			mTts.stop();
		}
		mPlayListIndex = HUGE;
		setButtonsVisibility();
	}
	
	public void destroy() {
		if (mTts != null)	
			mTts.shutdown();
		mTts = null;
	}

    public boolean isExpanded() {
    	return ttsMenuItem.isActionViewExpanded();
    }
    
    public boolean onMenuItemActionExpand(MenuItem item) {
        parent.setVolumeControlStream(AudioManager.STREAM_MUSIC);
        return parent.collapseActionViewsExcept(item);
    }
    
    public boolean onMenuItemActionCollapse(MenuItem item) {
        parent.setVolumeControlStream(AudioManager.USE_DEFAULT_STREAM_TYPE);
        /*if (mPopup.isShowing())
    		mPopup.dismiss();*/
        return true;
    }
    
    public void collapse (MenuItem exclude) {
        if (exclude != ttsMenuItem) {
            ttsMenuItem.collapseActionView();
        }
    }

	public void expandActionView() {
		ttsMenuItem.expandActionView();
	}

	@Override
	public void onUtteranceCompleted(String utteranceId) {
		mPlayListIndex++;
		readCurrent();
	}

	@SuppressWarnings("deprecation")
	@Override
	public void onInit(int status) {
        if (status == TextToSpeech.SUCCESS) {
        	mTts.setOnUtteranceCompletedListener(this);
        	loadPreferences();
            readTTS();
        } else {
            Toast.makeText(parent, R.string.tts_init_failed, Toast.LENGTH_LONG).show();
            mTts = null;
        }
	}
	
	public class TTSSettingsPopup extends IgnPopupWindow implements OnSeekBarChangeListener {
	    private View mDropDownView;
		private SeekBar mPitchSeekBar;
		private SeekBar mSpeedSeekBar;
		private Button mSettingsButton;
		
	    public TTSSettingsPopup(Context context) {
	        super(context);
	        
	        setAnchorView(settingsButton);
	        setModal(true);
	    }

	    @Override
	    public void show() {
	        final int spinnerPaddingLeft = settingsButton.getPaddingLeft();
	        setHorizontalOffset(spinnerPaddingLeft);
	        setInputMethodMode(PopupWindow.INPUT_METHOD_NOT_NEEDED);
	        super.show();
	    }

		@Override
		protected int buildDropDown() {
			if (mDropDownView == null) {
		        mDropDownView = new LinearLayout(mContext);
		        View.inflate(mContext, R.layout.tts_settings, (LinearLayout)mDropDownView);
			    
		        mDropDownView.setFocusable(true);
		        mDropDownView.setFocusableInTouchMode(true);
		        mPopup.setContentView(mDropDownView);
		        
		        setEventHandlers();
		        
		        mDropDownView.measure(MeasureSpec.UNSPECIFIED, MeasureSpec.UNSPECIFIED);
		        setContentWidth(mDropDownView.getMeasuredWidth());
		        setContentHeight(mDropDownView.getMeasuredHeight());
		    } else {
		    	mDropDownView = mPopup.getContentView();
		    }
		
		    // getMaxAvailableHeight() subtracts the padding, so we put it back
		    // to get the available height for the whole window
		    int padding = 0;
		    Drawable background = mPopup.getBackground();
		    if (background != null) {
		        background.getPadding(mTempRect);
		        padding = mTempRect.top + mTempRect.bottom;
		
		        // If we don't have an explicit vertical offset, determine one from the window
		        // background so that content will line up.
		        if (!mDropDownVerticalOffsetSet) {
		            mDropDownVerticalOffset = -mTempRect.top;
		        }
		    }
		
		    // Max height available on the screen for a popup.
		    boolean ignoreBottomDecorations =
		            mPopup.getInputMethodMode() == PopupWindow.INPUT_METHOD_NOT_NEEDED;
		    final int maxHeight = /*mPopup.*/getMaxAvailableHeight(
		            mDropDownAnchorView, mDropDownVerticalOffset, ignoreBottomDecorations);
		
		    if (mDropDownHeight == ViewGroup.LayoutParams.MATCH_PARENT) {
		        return maxHeight + padding;
		    }
		
			return mDropDownHeight + padding;
		}

		private void setEventHandlers() {
			mPitchSeekBar = (SeekBar)mDropDownView.findViewById(R.id.tts_pitch_seekbar);
			mSpeedSeekBar = (SeekBar)mDropDownView.findViewById(R.id.tts_speed_seekbar);
			
			mPitchSeekBar.setProgress(parent.sharedPreferences.getInt("ttsPitch", 3));
			mSpeedSeekBar.setProgress(parent.sharedPreferences.getInt("ttsSpeed", 3));
			
			mPitchSeekBar.setOnSeekBarChangeListener(this);
			mSpeedSeekBar.setOnSeekBarChangeListener(this);
			
			mSettingsButton = (Button)mDropDownView.findViewById(R.id.tts_settings_button);
			mSettingsButton.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					startTtsSettings();				
				}
			});
		}

		@Override
		public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
			SharedPreferences.Editor editor = parent.sharedPreferences.edit();

			if (seekBar == mPitchSeekBar)
				editor.putInt("ttsPitch", progress);
			else if (seekBar == mSpeedSeekBar)
				editor.putInt("ttsSpeed", progress);
			editor.commit();
			
			loadPreferences();
		}

		@Override
		public void onStartTrackingTouch(SeekBar seekBar) {}

		@Override
		public void onStopTrackingTouch(SeekBar seekBar) {}
	}

	private float f (int x, float m, float M) {
		float xnorm = (x - 3f) / 3;
		if (xnorm < 0)
			return xnorm * (1 - m) + 1;
		else if (xnorm > 0)
			return xnorm * (M - 1) + 1;
		else
			return 1;
	}
	
	public void loadPreferences() {
		if (mTts == null)
			return;
		
		int pitch = parent.sharedPreferences.getInt("ttsPitch", 3);
		int speed = parent.sharedPreferences.getInt("ttsSpeed", 3);
		
		mTts.setPitch(f(pitch, 0.5f, 2.0f));
		mTts.setSpeechRate(f(speed, 0.5f, 2.0f));
		
		mPlayListIndex--;
		readCurrent();
	}

	private void startTtsSettings() {
		Intent intent = new Intent();
		intent.setAction("com.android.settings.TTS_SETTINGS");
		//intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		parent.startActivity(intent);
		
		/*
		ComponentName componentToLaunch = new ComponentName(
		        "com.android.settings",
		        "com.android.settings.TextToSpeechSettings");
		Intent intent = new Intent();
		intent.addCategory(Intent.CATEGORY_LAUNCHER);
		intent.setComponent(componentToLaunch);
		intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		startActivity(intent);
		*/
	}

	private void startInstallLanguage() {
		Intent installIntent = new Intent();
		installIntent.setAction(TextToSpeech.Engine.ACTION_INSTALL_TTS_DATA);
		parent.startActivity(installIntent);
	}
}
