package net.laparola.ui.android.actionbar.view.ignspinner;

import net.laparola.core.Testi.TestoTipi;
import net.laparola.ui.android.actionbar.VersionAdapter;
import net.laparola.ui.android.actionbar.view.IgnPopupWindow;
import net.laparola.ui.android.actionbar.view.ignspinner.IgnDropDownListView;
import net.laparola.ui.android.actionbar.view.ignspinner.IgnDropDownListView.OnTypeChangedListener;

import com.actionbarsherlock.R;

import android.annotation.SuppressLint;
import android.content.Context;
import android.database.DataSetObserver;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewParent;
import android.view.View.MeasureSpec;
import android.widget.AdapterView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.PopupWindow;

/**
 * A proxy between pre- and post-Honeycomb implementations of this class.
 */
class IgnSpinnerPopupWindow extends IgnPopupWindow {
	class SpinnerResizePopupRunnable extends ResizePopupRunnable {
        public void run() {
            final ListView listView = mDropDownList.getListView();
			if (mDropDownList != null && listView.getCount() > listView.getChildCount() &&
					listView.getChildCount() <= mListItemExpandMaximum) {
                mPopup.setInputMethodMode(PopupWindow.INPUT_METHOD_NOT_NEEDED);
                show();
            }
        }
    }
	
	class SpinnerShowPopupRunnable extends ShowPopupRunnable {
        public void run() {
	        mDropDownList.setSelection(ListView.INVALID_POSITION);
	        mDropDownList.setSelectedType(mAdapter.getTipo());
	        
	        if (!mModal || mDropDownList.isInTouchMode()) {
	            clearListSelection();
	        }
	        
	        if (!mModal) {
	            mHandler.post(mHideSelector);
	        }
        }
    }
    
    IgnDropDownListView mDropDownList;

    AdapterView.OnItemClickListener mItemClickListener;
    AdapterView.OnItemSelectedListener mItemSelectedListener;
	private ListSelectorHider mHideSelector = new ListSelectorHider();
	private PopupScrollListener mScrollListener = new PopupScrollListener();
	private int mPromptPosition = POSITION_PROMPT_ABOVE;
	private Drawable mDropDownListHighlight;
	protected View mPromptView;

	public void setPromptPosition(int position) {
	    mPromptPosition = position;
	}

    public IgnSpinnerPopupWindow(Context context) {
        this(context, null, R.attr.listPopupWindowStyle);
    }

    public IgnSpinnerPopupWindow(Context context, AttributeSet attrs, int defStyleAttr) {
    	super(context, attrs, defStyleAttr);
    }

    @SuppressLint("NewApi")
	public IgnSpinnerPopupWindow(Context context, AttributeSet attrs, int defStyleAttr, int defStyleRes) {
    	super(context, attrs, defStyleAttr, defStyleRes);
    }

    public void setAdapter(VersionAdapter adapter) {
        if (mObserver == null) {
            mObserver = new PopupDataSetObserver();
        } else if (mAdapter != null) {
            mAdapter.unregisterDataSetObserver(mObserver);
        }
        mAdapter = adapter;
        if (mAdapter != null) {
            adapter.registerDataSetObserver(mObserver);
        }

        if (mDropDownList != null) {
            mDropDownList.setAdapter(mAdapter);
        }
    }

    public void setOnItemClickListener(AdapterView.OnItemClickListener clickListener) {
        mItemClickListener = clickListener;
    }

    public void clearListSelection() {
        final IgnDropDownListView list = mDropDownList;
        if (list != null) {
            // WARNING: Please read the comment where mListSelectionHidden is declared
            list.mListSelectionHidden = true;
            //XXX list.hideSelector();
            list.requestLayout();
        }
    }

    public ListView getListView() {
        return mDropDownList.getListView();
    }

    private class PopupDataSetObserver extends DataSetObserver {
        @Override
        public void onChanged() {
            if (isShowing()) {
                // Resize the popup to fit new content
                show();
            }
        }

        @Override
        public void onInvalidated() {
            dismiss();
        }
    }

    class ListSelectorHider implements Runnable {
        public void run() {
            clearListSelection();
        }
    }
    
    @Override
    public void dismiss() {
    	super.dismiss();
	    if (mPromptView != null) {
	        final ViewParent parent = mPromptView.getParent();
	        if (parent instanceof ViewGroup) {
	            final ViewGroup group = (ViewGroup) parent;
	            group.removeView(mPromptView);
	        }
	    }
	    if (mDropDownList != null) {
	        final ViewParent parent = mDropDownList.getParent();
	        if (parent instanceof ViewGroup) {
	            final ViewGroup group = (ViewGroup) parent;
	            group.removeView(mDropDownList);
	        }
    	    mDropDownList = null;
	    }
    }

    @Override
	protected int buildDropDown() {
	    ViewGroup dropDownView;
	    int otherHeights = 0;
	
	    if (mDropDownList == null) {
	        Context context = mContext;
	
	        mDropDownList = new IgnDropDownListView(context, !mModal);
	        if (mDropDownListHighlight != null) {
	            mDropDownList.setSelector(mDropDownListHighlight);
	        }
	        mDropDownList.setAdapter(mAdapter);
	        mDropDownList.setOnItemClickListener(mItemClickListener);
	        mDropDownList.setFocusable(true);
	        mDropDownList.setFocusableInTouchMode(true);
	        mDropDownList.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
	            public void onItemSelected(AdapterView<?> parent, View view,
	                    int position, long id) {
	
	                if (position != -1) {
	                    IgnDropDownListView dropDownList = mDropDownList;
	
	                    if (dropDownList != null) {
	                        dropDownList.mListSelectionHidden = false;
	                    }
	                }
	            }
	
	            public void onNothingSelected(AdapterView<?> parent) {
			    	dismiss();
	            }
	        });
	        mDropDownList.setOnScrollListener(mScrollListener);
	        mDropDownList.setOnTypeChangedListener(new OnTypeChangedListener() {
				@Override
				public void OnTypeChanged(TestoTipi tipo) {
			    	mAdapter.setTipo(tipo);
			    	dismiss();
					show();
				}
			});
	        
	        if (mItemSelectedListener != null) {
	            mDropDownList.setOnItemSelectedListener(mItemSelectedListener);
	        }
	
	        dropDownView = mDropDownList;
	
	        View hintView = mPromptView;
	        if (hintView != null) {
	            // if an hint has been specified, we accomodate more space for it and
	            // add a text view in the drop down menu, at the bottom of the list
	            LinearLayout hintContainer = new LinearLayout(context);
	            hintContainer.setOrientation(LinearLayout.VERTICAL);
	
	            LinearLayout.LayoutParams hintParams = new LinearLayout.LayoutParams(
	                    ViewGroup.LayoutParams.MATCH_PARENT, 0, 1.0f
	            );
	
	            switch (mPromptPosition) {
	            case POSITION_PROMPT_BELOW:
	                hintContainer.addView(dropDownView, hintParams);
	                hintContainer.addView(hintView);
	                break;
	
	            case POSITION_PROMPT_ABOVE:
	                hintContainer.addView(hintView);
	                hintContainer.addView(dropDownView, hintParams);
	                break;
	
	            default:
	                break;
	            }
	
	            // measure the hint's height to find how much more vertical space
	            // we need to add to the drop down's height
	            int widthSpec = MeasureSpec.makeMeasureSpec(mDropDownWidth, MeasureSpec.AT_MOST);
	            int heightSpec = MeasureSpec.UNSPECIFIED;
	            hintView.measure(widthSpec, heightSpec);
	
	            hintParams = (LinearLayout.LayoutParams) hintView.getLayoutParams();
	            otherHeights = hintView.getMeasuredHeight() + hintParams.topMargin + hintParams.bottomMargin;
	
	            dropDownView = hintContainer;
	        }
	
	        mPopup.setContentView(dropDownView);
	    } else {
	        dropDownView = (ViewGroup) mPopup.getContentView();
	        final View view = mPromptView;
	        if (view != null) {
	            LinearLayout.LayoutParams hintParams =
	                    (LinearLayout.LayoutParams) view.getLayoutParams();
	            otherHeights = view.getMeasuredHeight() + hintParams.topMargin
	                    + hintParams.bottomMargin;
	        }
	    }
	
	    // getMaxAvailableHeight() subtracts the padding, so we put it back
	    // to get the available height for the whole window
	    int padding = 0;
	    Drawable background = mPopup.getBackground();
	    if (background != null) {
	        background.getPadding(mTempRect);
	        padding = mTempRect.top + mTempRect.bottom;
	
	        // If we don't have an explicit vertical offset, determine one from the window
	        // background so that content will line up.
	        if (!mDropDownVerticalOffsetSet) {
	            mDropDownVerticalOffset = -mTempRect.top;
	        }
	    }
	
	    // Max height available on the screen for a popup.
	    boolean ignoreBottomDecorations =
	            mPopup.getInputMethodMode() == PopupWindow.INPUT_METHOD_NOT_NEEDED;
	    final int maxHeight = /*mPopup.*/getMaxAvailableHeight(
	            mDropDownAnchorView, mDropDownVerticalOffset, ignoreBottomDecorations);
	
	    if (mDropDownHeight == ViewGroup.LayoutParams.MATCH_PARENT) {
	        return maxHeight + padding;
	    }
	
	    final int listContent = mDropDownList.measureHeightOfChildren(MeasureSpec.UNSPECIFIED,
	            0, -1/*ListView.NO_POSITION*/, maxHeight - otherHeights, -1);
	    // add padding only if the list has items in it, that way we don't show
	    // the popup if it is not needed
	    if (listContent > 0) otherHeights += padding;
	
	    return listContent + otherHeights;
	}

	@Override
	protected void initRunnables() {
        mResizePopupRunnable = new SpinnerResizePopupRunnable();
    	mShowPopupRunnable = new SpinnerShowPopupRunnable();
	}
}
