package net.laparola.ui.android.library.downloadmanager;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import net.laparola.ui.LaParolaBrowser;
import net.laparola.ui.android.LaParolaActivity;
import net.laparola.ui.android.library.LibraryItemInfo;
import net.laparola.ui.utils.LZMAFile;
import net.laparola.ui.utils.lzma_java.LZMADecoder;
import android.app.Activity;
import android.app.Notification;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.os.PowerManager;

class DownloadTask extends AsyncTask<Void, Integer, Boolean> {
	/* package */ static final int DOWNLOAD_PERCENT = 80;

	public enum Status {
		WORKING, DONE, ERROR
	}

	private static final boolean USE_SD_REPOSITORY = false;

	private static final int BUFFER_LENGHT = 16384;

	private LibraryDownloaderService libraryDownloader;
	public int notificationID;
	public LibraryItemInfo info;
	public Status status;
	public int progress;

	public DownloadTask(LibraryDownloaderService libraryDownloader) {
		this.libraryDownloader = libraryDownloader;
		status = Status.WORKING;
	}

	protected Boolean doInBackground(Void... params) {
		publishProgress(0);
		
		SharedPreferences sharedPreferences = libraryDownloader.getSharedPreferences(LaParolaActivity.LAPAROLA_PREFERENCES, Activity.MODE_PRIVATE);
		return downloadAndUncompress(sharedPreferences.getBoolean("useLzma", false));
	}

	private boolean downloadAndUncompress(boolean useLzma) {
		try {
			download(useLzma);
		} catch (Exception e) {
			e.printStackTrace();
			if (useLzma == true) {
				return downloadAndUncompress(false);
			} else {
				return false;
			}
		} 

		String downloadFileType = useLzma ? info.getDownload1FileType() : info.getDownload2FileType();
		long downloadSize = useLzma ? info.getDownload1Size() : info.getDownload2Size();
		
		File comprFile = new File(info.getFileName() + "." + downloadFileType);
		if (comprFile.length() != downloadSize) {
			return false;
		}
		
		PowerManager pm = (PowerManager)libraryDownloader.getSystemService(Context.POWER_SERVICE);
		PowerManager.WakeLock wl = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, "laparola uncompress");
		wl.acquire();
		
		try {
			if (downloadFileType.equals("zip")) {
				uncompresszip(comprFile);
			} else {
				uncompresslzma(comprFile);
			}
		} catch (Exception e) {
			e.printStackTrace();
			if (useLzma) {
				return downloadAndUncompress(false);
			} else {
				return false;
			}
		} finally {
			wl.release();
			comprFile.delete();
		}

		LaParolaBrowser.cancellaTesto(info.getName(), info.getFileName());
		File filetmp = new File(info.getFileName() + ".tmp");
		File file = new File(info.getFileName());
		filetmp.renameTo(file);

		//try {
			LaParolaBrowser.aggiungiTesto(info.getFileName());
		//} catch (FileNonValidoException e) {
		//	return false;
		//}
			
		return true;
	}

	private void uncompresslzma(File comprFile) throws Exception {
		InputStream in;
		OutputStream out;
		try {
			in = new FileInputStream(comprFile);
			out = new FileOutputStream(info.getFileName() + ".tmp");
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

		BufferedInputStream src = new BufferedInputStream(in, BUFFER_LENGHT);
		BufferedOutputStream dest = new BufferedOutputStream(out, BUFFER_LENGHT);

		try {
			LZMAFile.decomprimi(src, dest, new LZMADecoder.ProgressRunnable() {
				int lastPercent = 0;
				long lastPercentTime = 0;

				@Override
				public void publish(long progresso, long size) {
					int percent = (int) Math.round(DOWNLOAD_PERCENT + (double) progresso / size * (100 - DOWNLOAD_PERCENT));
					long milliTime = System.nanoTime() / 1000000;
					if (lastPercent != percent && milliTime > lastPercentTime + 1000) {
						// android.util.Log.d("laparola", String.format("%d %d", progress, size));
						publishProgress(percent);
						lastPercent = percent;
						lastPercentTime = milliTime;
					}
				}
			});
        } catch (Exception e) {
			throw e;
		} finally {
			in.close();
			out.close();
		}
	}

	private void uncompresszip(File zipFile) throws IOException {
		ZipFile zip = new ZipFile(zipFile);
		ZipEntry zipEntry = zip.entries().nextElement();
		InputStream in = zip.getInputStream(zipEntry);
		OutputStream out = new FileOutputStream(info.getFileName() + ".tmp");

		long uncompressed = 0;
		long length = zipEntry.getSize();

		int lastProgress = -1;
		long lastProgressTime = -1;

		try {
			byte buf[] = new byte[BUFFER_LENGHT];
			int len;
			while (!isCancelled() && (len = in.read(buf)) > 0) {
				out.write(buf, 0, len);

				uncompressed += len;
				int progresso = (int) Math.round(DOWNLOAD_PERCENT + (double) uncompressed / length * (100 - DOWNLOAD_PERCENT));
				long milliTime = System.nanoTime() / 1000000;
				if (lastProgress != progresso && milliTime > lastProgressTime + 1000) {
					publishProgress(progresso);
					lastProgress = progresso;
					lastProgressTime = milliTime;
				}
			}
		} finally {
			in.close();
			out.close();
		}
	}

	private void download(boolean useLzma) throws MalformedURLException, IOException, FileNotFoundException {
		/*
		for (int progresso = 0; progresso < DOWNLOAD_PERCENT && !isCancelled(); progresso++) { 
			publishProgress(progresso);
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
			}
		}
		*/
		
		PowerManager pm = (PowerManager)libraryDownloader.getSystemService(Context.POWER_SERVICE);
		PowerManager.WakeLock wl = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, "laparola download");
		wl.acquire();
		
		InputStream fis = null;
		OutputStream fos = null;

		try {
			String url = useLzma ? info.getUrl() : info.getUrl2();
			if (USE_SD_REPOSITORY) {
				url = url.replace("http://", "file:///sdcard/laparola/");
			}

			URL u = new URL(url);
			URLConnection con = u.openConnection();
			con.setConnectTimeout(10000);
			con.setReadTimeout(10000);
			int contentLength = con.getContentLength();
			fis = con.getInputStream();

			File file = new File(info.getFileName() + "." + (useLzma ? info.getDownload1FileType() : info.getDownload2FileType()));

			file.delete();   // TODO : implementare ripresa download interrotti!

			fis.skip(file.length());
			long downloaded = file.length();
			fos = new FileOutputStream(file, file.exists());

			int lastProgress = -1;
			long lastProgressTime = -1;

			byte buf[] = new byte[BUFFER_LENGHT];
			int len;
			while (!isCancelled() && (len = fis.read(buf)) > 0) {
				fos.write(buf, 0, len);

				downloaded += len;
				int progresso = (int) Math.round((double) downloaded / contentLength * DOWNLOAD_PERCENT);
				long milliTime = System.nanoTime() / 1000000;
				if (lastProgress != progresso && milliTime > lastProgressTime + 1000) {
					publishProgress(progresso);
					lastProgress = progresso;
					lastProgressTime = milliTime;
				}

				if (USE_SD_REPOSITORY) {
					try {
						Thread.sleep(50);
					} catch (Exception e) {
						//
					}
				}
			}
		} finally {
	 		wl.release();
			if (fis != null)
				fis.close();
			if (fos != null)
				fos.close();
		}
	}

	@Override
	protected void onProgressUpdate(Integer... values) {
		progress = values[0];
		notificate(progress);
	}

	private void notificate(Integer progresso) {
		Notification notification = DownloadNotificationBuilder.getNotification(libraryDownloader, info, notificationID, progresso);
		libraryDownloader.notify(notificationID, notification);
	}

	protected void onPostExecute(Boolean result) {
		if (result == true) {
			status = Status.DONE;
			notificate(100);
		} else {
			status = Status.ERROR;
			notificate(-1);
		}
		libraryDownloader.onDownloadFinished(this, result);
	}

	protected void onCancelled() {
		libraryDownloader.cancelNotification(notificationID);
	}
}